@@ -1,4 +1,4 @@
-use super::{stars, ManiaDifficultyAttributes, ManiaPerformanceAttributes};
+use super::{ManiaDifficultyAttributes, ManiaPerformanceAttributes, ManiaStars};
 use crate::{Beatmap, DifficultyAttributes, Mods, PerformanceAttributes};
 
 /// Performance calculator on osu!mania maps.
@@ -36,6 +36,7 @@ pub struct ManiaPP<'map> {
     mods: u32,
     pub(crate) score: Option<f64>,
     passed_objects: Option<usize>,
+    clock_rate: Option<f64>,
 }
 
 impl<'map> ManiaPP<'map> {
@@ -48,6 +49,7 @@ impl<'map> ManiaPP<'map> {
             mods: 0,
             score: None,
             passed_objects: None,
+            clock_rate: None,
         }
     }
 
@@ -97,11 +99,31 @@ impl<'map> ManiaPP<'map> {
         self
     }
 
+    /// Adjust the clock rate used in the calculation.
+    /// If none is specified, it will take the clock rate based on the mods
+    /// i.e. 1.5 for DT, 0.75 for HT and 1.0 otherwise.
+    #[inline]
+    pub fn clock_rate(mut self, clock_rate: f64) -> Self {
+        self.clock_rate = Some(clock_rate);
+
+        self
+    }
+
     /// Calculate all performance related values, including pp and stars.
     pub fn calculate(self) -> ManiaPerformanceAttributes {
-        let stars = self
-            .stars
-            .unwrap_or_else(|| stars(self.map, self.mods, self.passed_objects).stars);
+        let stars = self.stars.unwrap_or_else(|| {
+            let mut calculator = ManiaStars::new(self.map).mods(self.mods);
+
+            if let Some(passed_objects) = self.passed_objects {
+                calculator = calculator.passed_objects(passed_objects);
+            }
+
+            if let Some(clock_rate) = self.clock_rate {
+                calculator = calculator.clock_rate(clock_rate);
+            }
+
+            calculator.calculate().stars
+        });
 
         let ez = self.mods.ez();
         let nf = self.mods.nf();
@@ -119,7 +141,7 @@ impl<'map> ManiaPP<'map> {
         }
 
         let mut od = 34.0 + 3.0 * (10.0 - self.map.od as f64).max(0.0).min(10.0);
-        let clock_rate = self.mods.speed();
+        let clock_rate = self.mods.clock_rate();
 
         let mut multiplier = 0.8;
 
